(function (global, factory) {
  typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :
  typeof define === 'function' && define.amd ? define(['exports'], factory) :
  (factory((global.svelteObservable = {})));
}(this, (function (exports) { 'use strict';

  var SVELTE_OBSERVABLE =
    typeof Symbol === 'function'
      ? Symbol('svelte-observable')
      : '@@svelte-observable';

  var OBSERVABLE;

  function isObservable(value) {
    // Lazy-load Symbol to give polyfills a chance to run
    if (!OBSERVABLE) {
      OBSERVABLE =
        (typeof Symbol === 'function' && Symbol.observable) || '@@observable';
    }

    return value && value[OBSERVABLE] && value[OBSERVABLE]() === value;
  }

  function deferred() {
    var resolve, reject;
    var later = new Promise(function (_resolve, _reject) {
      resolve = _resolve;
      reject = _reject;
    });

    later.resolve = resolve;
    later.reject = reject;

    return later;
  }

  function nonenumerable(target, name, value) {
    Object.defineProperty(target, name, {
      enumerable: false,
      configurable: true,
      writable: true,
      value: value
    });
  }

  function observe(value) {
    if (!isObservable(value)) { return value; }

    // Return a deferred Promise that will resolve to the initial observable value
    // and subsequently be replaced with additional values
    //
    // (wait until subscribe to kick off observable)
    var wrapped = deferred();
    nonenumerable(wrapped, SVELTE_OBSERVABLE, value);

    return wrapped;
  }

  function unsubscribe(component) {
    component = component || this;

    var subscriptions = component[SVELTE_OBSERVABLE];
    if (!subscriptions) { return; }

    for (var key in subscriptions) {
      subscriptions[key].unsubscribe();
    }

    nonenumerable(component, SVELTE_OBSERVABLE, null);
  }

  function subscribe(ref) {
    var this$1 = this;
    var changed = ref.changed;
    var current = ref.current;
    var previous = ref.previous;

    // Load subscriptions from component
    // (if none found, assume first run and attach destroy listener to unsubscribe)
    var subscriptions = this[SVELTE_OBSERVABLE];
    if (!subscriptions) {
      nonenumerable(this, SVELTE_OBSERVABLE, {});
      subscriptions = this[SVELTE_OBSERVABLE];

      this.on('destroy', function () { return unsubscribe(this$1); });
    }

    // Check for observables and subscribe / unsubscribe as-needed
    for (var key in changed) {
      var value = current[key];
      var observable = value && value[SVELTE_OBSERVABLE];
      var previous_observable =
        previous && previous[key] && previous[key][SVELTE_OBSERVABLE];

      if (observable && observable === previous_observable) {
        continue;
      }

      var existing = subscriptions[key];
      if (existing) {
        // Observable was overwritten, unsubscribe
        existing.unsubscribe();
      }

      if (observable) {
        subscriptions[key] = createSubscription(this$1, key, value);
      }
    }
  }

  function createSubscription(component, key, observer) {
    // Subscribe to changes
    // - First change, resolve initial deferred value
    // - Subsequent changes, set on component
    var observable = observer[SVELTE_OBSERVABLE];
    var resolved = false;

    var replace = function (value) {
      var obj;

      if (!resolved && observer.resolve) {
        resolved = true;
        observer.resolve(value);
      } else {
        // Continue exposing underlying observable on set values
        nonenumerable(value, SVELTE_OBSERVABLE, observable);
        component.set(( obj = {}, obj[key] = value, obj));
      }
    };

    var subscription = observable.subscribe({
      next: function (value) { return replace(Promise.resolve(value)); },
      error: function (error) { return replace(Promise.reject(error)); }
    });

    return subscription;
  }

  exports.SVELTE_OBSERVABLE = SVELTE_OBSERVABLE;
  exports.isObservable = isObservable;
  exports.deferred = deferred;
  exports.nonenumerable = nonenumerable;
  exports.observe = observe;
  exports.subscribe = subscribe;
  exports.unsubscribe = unsubscribe;

  Object.defineProperty(exports, '__esModule', { value: true });

})));
//# sourceMappingURL=svelte-observable.umd.js.map
